import React from 'react';
import {createBottomTabNavigator} from '@react-navigation/bottom-tabs';
import * as Font from 'expo-font';
import {AppLoading, registerRootComponent} from "expo";
import {Ionicons} from '@expo/vector-icons';
import {NavigationContainer} from '@react-navigation/native';
import {InitialProps} from "expo/build/launch/withExpoRoot.types";
import {StyleConstants} from "./style/StyleConstants";
import User from "./model/User";
import CredentialsService from "./services/InternalStoreService";
import {LoginScreen} from "./screens/LoginScreen";
import {navigationRef} from "./navigation/RootNavigation";
import {AuthContext, AuthContextInterface} from 'context/AuthContext';
import {CatalogStack} from "./navigation/CatalogStack";
import {HomeStack} from "./navigation/HomeStack";
import {FeedScreen} from "./screens/FeedScreen";
import {InfoScreen} from "./screens/InfoScreen";


enum AuthActions {
    SIGN_IN = 'SIGN_IN',
    LOADING_FINISHED = 'LOADING_FINISHED',
    USER_ALREADY_SIGNED = 'USER_ALREADY_SIGNED',
    SIGN_OUT = 'SIGN_OUT',
}

interface State {
    userSigned: boolean,
    isLoading: true,
}

export const App : React.FC<InitialProps> = (initialProps: InitialProps) => {

    const [state, dispatch] = React.useReducer(
        (prevState: State, action: AuthActions) => {
            switch (action) {
                case AuthActions.SIGN_IN:
                    return {
                        ...prevState,
                        userSigned: true
                    };
                case AuthActions.LOADING_FINISHED:
                    return {
                        ...prevState,
                        isLoading: false,
                    };
                case  AuthActions.USER_ALREADY_SIGNED:
                    return {
                        ...prevState,
                        userSigned: true
                    };
                case  AuthActions.SIGN_OUT:
                    console.log("signing out");
                    return {
                        ...prevState,
                        userSigned: false
                    };

            }
        },
        {
            //initial state
            isLoading: true,
            userSigned: false
        }
    );

    const Loading = () => {
        const handleFinishLoading = () => {
            dispatch(AuthActions.LOADING_FINISHED);
        };

        const loadResourcesAsync = async () => {
            await Font.loadAsync({
                "roboto": require("./assets/fonts/Roboto-Regular.ttf"),
                "roboto-bold": require("./assets/fonts/Roboto-Bold.ttf"),
            })
        };

        const handleLoadingError = (error: Error) => {
            console.warn(error);
        };

        return (
            <AppLoading
                startAsync={loadResourcesAsync}
                onError={handleLoadingError}
                onFinish={handleFinishLoading}
            />
        )
    };

    React.useEffect(() => {
        // Fetch the token from storage then navigate to our appropriate place
        const bootstrapAsync = async () => {
            let user: User;

            try {
                user = await CredentialsService.getUser();
                //todo provolat api jestli si nezmenil napriklad heslo
                if(user.password) {
                    console.log("already signed");
                    dispatch(AuthActions.USER_ALREADY_SIGNED);
                }
            } catch (e) {
                // Restoring token failed
            }
            // After restoring token, we may need to validate it in production apps
            // This will switch to the App screen or Auth screen and this loading
            // screen will be unmounted and thrown away.
        };

        bootstrapAsync();
    }, []);

    const authContextImpl = React.useMemo<AuthContextInterface>(
        () => ({
            signIn: async (user: User) => {
                // In a production app, we need to send some data (usually username, password) to server and get a token
                // We will also need to handle errors if sign in failed
                // After getting token, we need to persist the token using `AsyncStorage`
                // In the example, we'll use a dummy token
                CredentialsService.saveUser(user);
                dispatch(AuthActions.SIGN_IN);
            },
            signOut: async () => {
                CredentialsService.deleteUser();
                dispatch(AuthActions.SIGN_OUT);
            },
            //todo ulozit usera
        }), []
    );

    const Tab = createBottomTabNavigator();

    return (
        <AuthContext.Provider value={authContextImpl}>
            <NavigationContainer theme={MyTheme} ref={navigationRef}>
                <Tab.Navigator screenOptions={({route}) => ({
                    tabBarIcon: ({focused, color, size}) => {
                        let iconName;
                        switch (route.name) {
                            case 'Info' : iconName= 'ios-information-circle'; break;
                            case 'Katalog' : iconName = 'ios-search'; break;
                            case 'Home' : iconName = 'ios-home'; break;
                            case 'Aktuality' : iconName='logo-rss'; break;
                        }

                        // You can return any component that you like here!
                        return <Ionicons name={iconName} size={size} color={color}/>;
                    },
                })}
                >
                    {state.isLoading && <Tab.Screen name={"Loading"} component={Loading} options={{tabBarVisible: false}}/>}
                    {!state.isLoading && !state.userSigned && <Tab.Screen name={"Login"} component={LoginScreen} options={{tabBarVisible: false}}/>}
                    {state.userSigned && <Tab.Screen name="Home" component={HomeStack}/>}
                    {state.userSigned && <Tab.Screen name="Katalog" component={CatalogStack}/>}
                    {state.userSigned && <Tab.Screen name="Aktuality" component={FeedScreen}/>}
                    {state.userSigned && <Tab.Screen name="Info" component={InfoScreen}/>}
                </Tab.Navigator>
            </NavigationContainer>
        </AuthContext.Provider>
    );
};

registerRootComponent(App);

export const MyTheme = {
    dark: true,
    colors: {
        primary: StyleConstants.PRIMARY_BLUE_COLOR,
        background:  StyleConstants.BACKGROUND,
        card:  StyleConstants.CARD,
        text: StyleConstants.TEXT,
        border:  StyleConstants.BORDER,
    },
};
