import React from "react";
import {ActivityIndicator, FlatList, Keyboard, StyleSheet, View,} from "react-native";
import {ButtonGroup, Input} from "react-native-elements";
import Constants from 'expo-constants';
import {RobotoBoldText} from "components/text/StyledText";
import CatalogBook from "model/CatalogBook";
import FindResult from "model/FindResult";
import AlephService from "services/AlephService";
import BookListItem from "components/BookListItem";
import AwesomeDebouncePromise from "awesome-debounce-promise";
import {StyleConstants} from "style/StyleConstants";
import * as RootNavigation from 'navigation/RootNavigation';
import {GLOBAL_STYLES} from "style/GlobalStyles";
import {CatalogScreens} from "navigation/enums/CatalogScreens";

interface Props {
}

interface State {
    loading: boolean;
    selectedFilter: number,
    input: String,
    books: Array<CatalogBook>,
    noResults: boolean
}

export default class AlephCatalogScreen extends React.Component<Props, State> {
    private readonly loadBy = 12;
    private findResult : FindResult = new FindResult();

    constructor(props: Props) {
        super(props);
        this.state = {
            loading: false,
            selectedFilter: 0,
            input: "",
            books: [ ],
            noResults: false
        };    
    }

    searchAPI = (input, filterIndex) => AlephService.findBooks(input, filterIndex, this.loadBy);

    searchAPIDebounced = AwesomeDebouncePromise(this.searchAPI, 500);

    updateFilter = (filterIndex: number) => {
        this.setState({loading: true, noResults: false, selectedFilter: filterIndex,  books: []},
            () => this.searchBooks(this.state.input));
    };

    searchBooks = async (input: String) => {
        if(input === '') {
            this.setState({loading: false, books: []})
        } else {
            this.setState(
                {loading: true, noResults: false, input: input, books: []}
            );
            try {
                this.findResult = await this.searchAPIDebounced(input, this.state.selectedFilter);
                if (this.findResult.numberOfRecords === 0) {
                    this.setState({loading: false, noResults: true, books: []});
                } else {
                    let data: CatalogBook[] = this.findResult.books;
                    this.setState({loading: false, noResults: false, books: this.state.books.concat(data)});
                }
            } catch (e) {
                console.log(e);
                this.setState({loading: false, books: []});
            }
        }
    };

    loadMoreBooks = async () => {
        if(this.findResult.numberOfRecords > this.state.books.length) {
            this.setState({loading: true});        
            let data = await AlephService.presentBooks(this.findResult, this.state.books.length + 1, this.loadBy);
            this.setState({loading: false, books: this.state.books.concat(data)});
        }
    };


    componentDidMount(): void {
        this.searchBooks('fyzika');
    }

    render() {
        return (
          <View style={styles.container}>
            <Input
                leftIcon={{name: 'search', color: StyleConstants.PRIMARY_BLUE_COLOR}}
              placeholder="Hledaný výraz"
              maxLength={20}
              onBlur={Keyboard.dismiss}
              onChangeText={text => {
                this.searchBooks(text);
              }}
              placeholderTextColor={StyleConstants.PRIMARY_BLUE_COLOR}
              inputStyle={[GLOBAL_STYLES.input, {marginLeft: 8}]}
              labelStyle={GLOBAL_STYLES.input}
              autoCapitalize="none"
            />
            <ButtonGroup
              onPress={this.updateFilter}
              selectedIndex={this.state.selectedFilter}
              buttons={[
                { element: () => <RobotoBoldText style={{color: "black"}}>Název</RobotoBoldText> },
                { element: () => <RobotoBoldText style={{color: "black"}}>Autor</RobotoBoldText> },
                { element: () => <RobotoBoldText style={{color: "black"}}>Signatura</RobotoBoldText> }
              ]}
              buttonStyle={
                  {
                      // backgroundColor: StyleConstants.PRIMARY_BLUE_COLOR
                  }
              }
              selectedButtonStyle={{
                  backgroundColor: StyleConstants.PRIMARY_BLUE_COLOR,
              }}
              selectedTextStyle={{
                  color: StyleConstants.TEXT
              }}
              containerStyle={styles.buttonGroup}
            />
            {this.state.input.length === 0 && (
              <RobotoBoldText style={styles.hintText}>
                Začněte psát pro vyhledávání
              </RobotoBoldText>
            )}
            {this.state.noResults && (
              <RobotoBoldText style={styles.hintText}>
                Nebyly nalezeny žádné záznamy
              </RobotoBoldText>
            )}
            {this.state.input.length > 0 && (
              <FlatList<CatalogBook>
                  style={{width: '100%'}}
                data={this.state.books}
                ItemSeparatorComponent={this.flatListItemSeparator}
                renderItem={this.renderItem}
                keyExtractor={this.keyExtractor}
                onEndReached={this.loadMoreBooks}
                onEndReachedThreshold={2}
                ListFooterComponent={this.listFooterComponent}
                refreshing={this.state.loading}
                />
                )}
          </View>
        );
    }



    renderItem = ({ item }: { item: CatalogBook }) => (
        <BookListItem title={item.title} author={item.author} year={item.year} isbn={item.isbn} onPress={() => RootNavigation.navigate(CatalogScreens.DETAIL, {book: item})} />
    );

    keyExtractor =  (item, index) => `${index}`;

    listFooterComponent = () => {
        if (this.state.loading) {
            return <ActivityIndicator style={styles.indicator} size="small" color={StyleConstants.PRIMARY_BLUE_COLOR}/>
        } else {
            return null;
        }
    };

    flatListItemSeparator = () => {
        return (
            <View style={styles.itemSeparator}/>
        );
    };
}


const styles = StyleSheet.create({
    container: {
        paddingTop: Constants.statusBarHeight,
        flex: 1,
        flexDirection: "column",
        // justifyContent: 'center',
        alignItems: 'center',
        backgroundColor: StyleConstants.BACKGROUND,
    },
    row: {
        flexDirection: "row",
        justifyContent: "center",
        alignItems: "center"
    },
    buttonGroup: {
        height: 50
    },
    hintText: {
        color: StyleConstants.PRIMARY_BLUE_COLOR,
        fontSize: 16,
        justifyContent: "center",
        alignItems: "center",
        alignContent: "center",
        marginTop: 10
    },
    indicator: {
        flex: 1,
        flexDirection: 'column',
        alignItems: 'center',
        justifyContent: 'center',
        paddingVertical: 10
    },
    itemSeparator: {
        height: 15,
        // width: '100%',
    }
});
