import React from "react";
import {ActivityIndicator, TouchableHighlight, View} from "react-native";
import {Icon} from "react-native-elements";
import {AuthContext, AuthContextInterface} from "../context/AuthContext";
import {MainView} from "../components/view/MainView";
import {RobotoBoldText} from "../components/text/StyledText";
import {useFocusEffect, useNavigation} from "@react-navigation/native";
import {StackNavigationProp} from "@react-navigation/stack";
import {HomeScreens} from "../navigation/enums/HomeScreens";
import User from "../model/User";
import CredentialsService from "../services/InternalStoreService";
import AlephService from "../services/AlephService";
import LoanBook from "../model/LoanBook";
import ReservationBook from "../model/ReservationBook";
import CatalogBook from "../model/CatalogBook";
import {StyleConstants} from "../style/StyleConstants";

type Props = {}

interface State {
    reservations: ReservationBook[],
    loans: LoanBook[],
    history: CatalogBook[],
    loading: boolean
}

export const HomeScreen: React.FC<Props> = () => {

    let userId: string;

    const [state, setState] = React.useState<State>({loading: false,reservations: [], loans: [], history: []});

    const navigation = useNavigation<StackNavigationProp<any>>();
    const {signOut} = React.useContext<AuthContextInterface>(AuthContext);


    const load = async () => {
        setState({...state, loading: true});
        let user: User = await CredentialsService.getUser();
        user = await AlephService.getUserInfo(user.username, user.password);
        userId = user.id;
        setState({loading: false, reservations: user.reservations, loans: user.loans, history: user.history});
    };

    useFocusEffect(
        //todo dat to spis na normalni pull to refresh a ne na focus
        React.useCallback(() => {
            // Do something when the screen is focused
            console.log("loading");
            load();
            return () => {
                // Do something when the screen is unfocused
                // Useful for cleanup functions
            };
        }, [])
    );

    const getTextBook = (array: CatalogBook[]) => {
        if(array && array.length > 0) {
            if(array.length > 1 && array.length < 5) {
                return array.length + " knihy";
            } else if(array.length >= 5){
                return array.length + " knih";
            } else {
                return array.length + " kniha";
            }
        } else {
            return "0 knih";
        }
    };

    return (
        <MainView style={{margin: 8}}>
            <View style={{flexDirection: "row", justifyContent: "space-between", marginBottom: 20}}>
                <RobotoBoldText h2 style={{color: StyleConstants.PRIMARY_BLUE_COLOR}}>Vítejte</RobotoBoldText>
                <Icon name="logout" type="material-community" onPress={() => signOut()} color={StyleConstants.PRIMARY_BLUE_COLOR} containerStyle={{alignContent: "flex-end", marginRight: 18, marginTop: 16}}/>
            </View>
            {state.loading && <ActivityIndicator/>}


            <TouchableHighlight style={{backgroundColor: StyleConstants.CARD_BACKGROUND, marginBottom: 5, borderWidth: 1, borderRadius: 10}}
                                onPress={()=> navigation.navigate(HomeScreens.LOANS, {loans: state.loans})}>
                <View style={{flexDirection: "row", margin: 8, justifyContent: 'space-between'}}>
                    <View>
                        <RobotoBoldText h4 style={{color: StyleConstants.PRIMARY_BLUE_COLOR}}>Výpůjčky</RobotoBoldText>
                        <RobotoBoldText style={{color: StyleConstants.PRIMARY_BLUE_COLOR}}>{getTextBook(state.loans)}</RobotoBoldText>
                    </View>

                    <Icon name='book' color={StyleConstants.PRIMARY_BLUE_COLOR} size={40} containerStyle={{padding: 4}}/>
                </View>
            </TouchableHighlight>

            <TouchableHighlight style={{backgroundColor: StyleConstants.CARD_BACKGROUND, marginBottom: 5, borderWidth: 1, borderRadius: 10}}
                                onPress={()=> navigation.navigate(HomeScreens.RESERVATIONS, {reservations: state.reservations})}>
                <View style={{flexDirection: "row", margin: 8, justifyContent: 'space-between'}}>
                    <View>
                        <RobotoBoldText h4 style={{color: StyleConstants.PRIMARY_BLUE_COLOR}}>Rezervace</RobotoBoldText>
                        <RobotoBoldText style={{color: StyleConstants.PRIMARY_BLUE_COLOR}}>{ getTextBook(state.reservations)}</RobotoBoldText>
                    </View>

                    <Icon name='md-calendar' type="ionicon" color={StyleConstants.PRIMARY_BLUE_COLOR} size={40} containerStyle={{padding: 7}}/>
                </View>
            </TouchableHighlight>

            <TouchableHighlight style={{backgroundColor: StyleConstants.CARD_BACKGROUND, borderWidth: 1, borderRadius: 10}}
                                onPress={()=> navigation.navigate(HomeScreens.HISTORY, {history: state.history})}>
                <View style={{flexDirection: "row", margin: 8, justifyContent: 'space-between'}}>
                    <View>
                        <RobotoBoldText h4 style={{color: StyleConstants.PRIMARY_BLUE_COLOR}}>Historie výpůjček</RobotoBoldText>
                        <RobotoBoldText style={{color: StyleConstants.PRIMARY_BLUE_COLOR}}>{ getTextBook(state.history) }</RobotoBoldText>
                    </View>

                    <Icon name='history' type={"font-awesome"} color={StyleConstants.PRIMARY_BLUE_COLOR} size={40} containerStyle={{padding: 4}}/>
                </View>
            </TouchableHighlight>


        </MainView>
    );
};