import CatalogBook from "model/CatalogBook";
import FindResult from "model/FindResult";
import Item from "model/Item";
import Constants from "expo-constants";
import User from "model/User";
import CredentialsService from "./InternalStoreService";
import InternalStoreService from "./InternalStoreService";
import FeedCard from "model/FeedCard";
import {Alert} from "react-native";

const { manifest } = Constants;

export default class AlephService {

  private static readonly API_URL = (typeof manifest.packagerOpts === `object`) && manifest.packagerOpts.dev
    ? manifest.debuggerHost.split(`:`).shift().concat(`:9090`)
    : `193.84.47.54:9090`;

  public static async processApiCall(url: string, init?: RequestInit): Promise<Response> {
    let response = await fetch(url, await this.getRequestWithAuthorization(init));
    if(response.status === 401 || response.status === 500) {
      //mozna vyprsel token
      let user = await InternalStoreService.getUser();
      if(user && this.authenticateUser(user.username, user.password)) {
        return await fetch(url, await this.getRequestWithAuthorization(init));
      } else {
        Alert.alert("Přihlášení", "Nepodařilo se ověřit přihlašovací údaje, zkuste se prosím znovu přihlásit");
      }
    } else {
      return response;
    }
  }

  public static async getRequestWithAuthorization(init?: RequestInit): Promise<RequestInit> {
    if(!init) {
      return {
        headers: {
          'Authorization': 'Bearer ' + await InternalStoreService.getToken()
        }
      }
    } else {
      return  {
        method: init.method,
        headers: {
          ...init.headers,
          'Authorization': 'Bearer ' + await InternalStoreService.getToken(),
        },
        body: init.body
      };
    }
  }

  public static async authenticateUser(username: String, password: String): Promise<boolean> {
    let response = await fetch(`http://${AlephService.API_URL}/authenticate`,
        {
          method: 'POST',
          headers: {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
          },
          body: JSON.stringify({username: username, password: password})
        }
    );
    let tokenJson = await response.json();
    if(tokenJson.token) {
      await InternalStoreService.saveToken(tokenJson.token);
      return true;
    } else {
      return false;
    }
  }

  public static async getUserInfo(username: String, password: String): Promise<User> {
    //zkontrolovat token
    const token = await InternalStoreService.getToken();
    if(!token || token.length == 0 ) {
      await this.authenticateUser(username, password);
    }

    let response = await this.processApiCall(`http://${AlephService.API_URL}/aleph/userInfo`,
        {
          method: 'POST',
          headers: {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
          },
          body: JSON.stringify({username: username, password: password} )
        }
    );
    return await response.json();
  }

  public static async findBooks(input: String, filterIndex: number, loadBy: number): Promise<FindResult> {
    let filter = "";
    switch (filterIndex) {
      case 0:
        filter = "title";
        break;
      case 1:
        filter = "author";
        break;
      case 2:
        filter = "signature";
        break;
    }
    input = input.split(" ").join("%20");
    console.info(`hledam: ${input} s filtrem: ${filter}; loadBy: ${loadBy}`);

    try {
      // console.log(`calling http://${AlephService.localAPI}/aleph/find?query=${input}&filter=${filter}`);
      let response = await this.processApiCall(`http://${AlephService.API_URL}/aleph/find?query=${input}&filter=${filter}`);
      let findResult: FindResult = await response.json();
      findResult.books = await this.presentBooks(findResult, 1, loadBy);
      return findResult;
    } catch (e) {
      console.log(e);
      Alert.alert("Načítání knih","Došlo k chybě při načítání knih");
    }
  }

  public static async presentBooks(findResult: FindResult, from: number, loadBy: number): Promise<CatalogBook[]> {
    try {
      let response = await this.processApiCall(`http://${AlephService.API_URL}/aleph/present?setNumber=${findResult.setNumber}&from=${from}&loadBy=${loadBy}`);
      return await response.json();
    } catch (e) {
      console.log(e);
      Alert.alert("Načítání knih","Došlo k chybě při načítání knih");
    }

  }

  public static async getItemData(docNumber: String): Promise<Item[]> {
    try {
      let response = await this.processApiCall(`http://${AlephService.API_URL}/aleph/itemData?docNumber=${docNumber}`);
      return await response.json();
    } catch (e) {
      console.log(e);
      Alert.alert("Načítání exemplářů","Došlo k chybě při načítání exemplářů");
    }
  }

  public static async holdRequest(docNumber: String, itemBarCode: String) {
    const user: User = await CredentialsService.getUser();
    return await this.processApiCall(`http://${AlephService.API_URL}/aleph/holdRequest?docNumber=${docNumber}&itemBarCode=${itemBarCode}&borId=${user.id}`);
  }

  public static async cancelHoldRequest(docNumber: String, itemSequence: String, sequence: String) {
      const response = await this.processApiCall(`http://${AlephService.API_URL}/aleph/cancelHoldRequest?docNumber=${docNumber}&itemSequence=${itemSequence}&sequence=${sequence}`);
      return response;
  }

  public static async renewLoan(itemBarCode: String, borId: String) {
    const response = await this.processApiCall(`http://${AlephService.API_URL}/aleph/renewLoan?itemBarCode=${itemBarCode}&borId=${borId}`);
    return response;
  }

  public static async getFeed(): Promise<FeedCard[]> {
    let response = await this.processApiCall(`http://${AlephService.API_URL}/aleph/feed`);
    return await response.json();
  }

}